/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <cassert>
#include <numeric>    // std::accumulate
#include <functional> // std::multiplies

#include "aidge/operator/Slice.hpp"

#include "aidge/backend/cpu/operator/SliceImpl.hpp"
#include "aidge/backend/cpu/operator/SliceImpl_forward_kernels.hpp"
#include "aidge/utils/Types.h"
#include <vector>
#include <cassert>
#include <tuple>

void Aidge::SliceImpl_cpu::forward() {
    // FIXME: uncomment the following code once memory handling will work
    assert(std::static_pointer_cast<Tensor>(mOp.getRawInput(0)) && "missing input #0");

    // Find the correct kernel type
    auto kernelFunc = Registrar<SliceImplForward_cpu>::create(
            {std::static_pointer_cast<Tensor>(mOp.getRawInput(0))->dataType()});

    // Call kernel
    kernelFunc(dynamic_cast<const Slice_Op&>(mOp).getStaticAttributes(),
            std::static_pointer_cast<Tensor>(mOp.getRawInput(0))->dims(),
            std::static_pointer_cast<Tensor>(mOp.getRawInput(0))->getImpl()->rawPtr(),
            std::static_pointer_cast<Tensor>(mOp.getRawOutput(0))->getImpl()->rawPtr()
            );
}

void Aidge::SliceImpl_cpu::backward() { fmt::print("Not implemented yet.\n"); }
